@@ -336,6 +336,14 @@ static void spl_set_combobox(GtkComboBox *cb, const ComboEntry centries[], guint
 
 }
 
+static void spl_name_entry_changed(GtkEntry *entry, GtkWidget *button) {
+    g_return_if_fail (button);
+    if (gtk_entry_get_text_length (entry) >= 1)
+        gtk_widget_set_sensitive(button, TRUE);
+    else
+        gtk_widget_set_sensitive(button, FALSE);
+}
+
 static void spl_matchcheckedonly_toggled(GtkToggleButton *togglebutton, GtkWidget *spl_window) {
     Playlist *spl;
 
@@ -1386,6 +1394,9 @@ static void spl_edit_all(iTunesDB *itdb, Playlist *spl, gint32 pos) {
 
     /* Set checkboxes and connect signal handlers */
     if ((w = gtkpod_builder_xml_get_widget(spl_wizard->builder, "spl_name_entry"))) {
+        g_signal_connect (w, "changed",
+                G_CALLBACK (spl_name_entry_changed),
+                gtkpod_builder_xml_get_widget(spl_wizard->builder, "spl_ok_button"));
         if (spl_dup->name)
             gtk_entry_set_text(GTK_ENTRY (w), spl_dup->name);
     }
@@ -1504,30 +1515,30 @@ void add_new_pl_or_spl_user_name(iTunesDB *itdb, gchar *dflt, gint32 position) {
         return;
     }
 
+    name = dflt ? dflt : _("New Playlist");
+retry:
     name
-            = get_user_string(_("New Playlist"), _("Please enter a name for the new playlist"), dflt ? dflt : _("New Playlist"), _("Smart Playlist"), &is_spl, GTK_STOCK_ADD);
+            = get_user_string(_("New Playlist"), _("Please enter a name for the new playlist"), name, _("Smart Playlist"), &is_spl, GTK_STOCK_ADD);
     if (!name)
         return;
 
     if (strlen(name) == 0) {
         gtkpod_warning_simple(_("Playlist name cannot be blank"));
-        return;
+        goto retry;
     }
 
     Playlist *pl = itdb_playlist_by_name(itdb, name);
     if (pl) {
         gtkpod_warning_simple(_("A playlist named '%s' already exists"), name);
-        return;
+        goto retry;
     }
 
-    if (name) {
-        if (!is_spl) { /* add standard playlist */
-            gp_playlist_add_new(itdb, name, FALSE, position);
-            gtkpod_tracks_statusbar_update ();
-        }
-        else { /* add smart playlist */
-            spl_edit_new(itdb, name, position);
-        }
+    if (!is_spl) { /* add standard playlist */
+        gp_playlist_add_new(itdb, name, FALSE, position);
+        gtkpod_tracks_statusbar_update ();
+    }
+    else { /* add smart playlist */
+        spl_edit_new(itdb, name, position);
     }
 }
 
