@@ -38,6 +38,7 @@
 #include <libanjuta/resources.h>
 #include <libanjuta/anjuta-plugin-manager.h>
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/anjuta-version.h>
 
 #include "anjuta-app.h"
 #include "anjuta-actions.h"
@@ -501,7 +502,11 @@ static void anjuta_app_instance_init(AnjutaApp *app) {
     g_list_free(plugins_dirs);
 
     /* Preferences */
+#if ANJUTA_CHECK_VERSION(3,5,3)
+    app->preferences = anjuta_preferences_new(app->plugin_manager, PREF_SCHEMA);
+#else
     app->preferences = anjuta_preferences_new(app->plugin_manager);
+#endif
     g_object_add_weak_pointer(G_OBJECT (app->preferences), (gpointer) &app->preferences);
 
     g_signal_connect(app->settings, "changed::" GDL_STYLE, G_CALLBACK(on_gdl_style_changed), app);
@@ -650,7 +655,7 @@ void anjuta_app_set_geometry(AnjutaApp *app, const gchar *geometry) {
         width = gdk_screen_width() - 10;
         height = gdk_screen_height() - 25;
         width = (width < 790) ? width : 790;
-        height = (height < 575) ? width : 575;
+        height = (height < 575) ? height : 575;
         if (gtk_widget_get_realized (GTK_WIDGET (app)) == FALSE) {
             gtk_window_set_default_size(GTK_WINDOW (app), width, height);
             gtk_window_move(GTK_WINDOW (app), posx, posy);
